<?php

/* NoPro handy-lib
 * some handy defs and funcs
 */

// version 2015-03-18
/***
 * 2015-08-18 added:
 * function nop_is_serialized($s) 
 * function nop_subst_serialized($ptrn, $subs, $data) 
 *
 * 2015-09-17 added:
 * function salt()
 *
 * 2015-09-23 xfind(): added parameter offset
 * 2016-02-20 added function showdiffs
 * 2016-02-27 added function dbg
 * 2016-04-04 added functions hostname and domainname
 * 2016-04-05 added functions linodoc
 */

if(!defined('CR')) define('CR', "\r");
if(!defined('LF')) define('LF', "\n");
if(!defined('CRLF')) define('CRLF', CR.LF);

function sqltimedate($date) {
  return date("Y-m-d H:i:s", strtotime($date));
}

function arr2tbl($arr, $cap) {
  // output html-table from array of assoc-arrays
  echo '  <table class="sqlres">' .CRLF;
  echo "    <caption>$cap</caption>" .CRLF;
  echo "    <tr>";
  foreach($arr[0] as $k => $v) {
    echo "      <th>". htmlify_sec($k) ."</th>" .CRLF;
  }
  echo "    </tr>" .CRLF;
  foreach($arr as $row) {
    echo "    <tr>" .CRLF;
    foreach($row as $k => $v) {
      echo "      <td>". htmlify_sec($v) ."</td>".CRLF;
    }
    echo "    </tr>" .CRLF;
  }
  echo "  </table>" .CRLF;
}

function npget_http($url, $chunked=false) {
  if(0 && !$chunked && function_exists('http_get')) {
    // use pecl http-extension
    $html = http_get($url);
  } else {
    // use curl
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_HEADER, 1);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    //curl_setopt($ch, CURLOPT_NOBODY, 0);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
    curl_setopt($ch, CURLOPT_MAXREDIRS, 2);
    curl_setopt($ch, CURLOPT_COOKIEFILE, '');
    curl_setopt($ch, CURLOPT_COOKIEJAR, '');
    curl_setopt($ch, CURLOPT_TIMEOUT, 25);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
    $resp = curl_exec($ch);
    curl_close($ch);
  }
  return $resp;
}

function npget_mywebip($k = 0) {
  $pool = array(
		# array("myip.sonyonline.net/", '-- You Are ', true),		// no regex
		# array("myip.sonyonline.net/", "/-- You Are \r\n../"),		// chunks 30 + ea + 1d + 19 + 3b but needs regex
		## array("myip.dk", 'IP Address:</b> <span class="ha4">'),	// 6024 not working anymore since 2012-09
		## array("myip.utilities-online.info/", "My ip is: "),		// chunks long  not working anymore since 2012-11
		# array('whatismyipaddress.com/', '<!-- do not script -->'),	// next line
		# array("www.ip-adress.com/IP_adresse/", "Meine IP: "),		// len: 5853 *but slow*
		array("myip.nmonitoring.com/", "<a href='http://whois.nmonitoring.com?ip="),	// chunks VERY long
	       );
  if($k<0 || $k>count($pool)) $k=0;
  list($site, $ptrn, $chunked) = $pool[$k];

  $html = npget_http("http://$site", $chunked);  
  #die($html);
  $pl = strlen($ptrn);
  $bp = stripos($html, $ptrn);
  $bp += $pl + strspn($html, "\r\n\t ", $bp + $pl);
  $len = strspn($html, ".0123456789", $bp);
  return substr($html, $bp, $len);
}

function htmlify($s) {
  // replace umlauts only
  // and a very few accentuated
  $patt = array('//',    '//',    '//',    '//',     '//',    '//',    '//',      '//'     , '//'    , '//');
  $repl = array('&auml;', '&ouml;', '&uuml;', '&szlig;', '&Auml;', '&Ouml;', '&Uuml;',   '&eacute;', '&ocirc;', '&iacute;');
  return preg_replace($patt, $repl, $s);
}

function htmlify_sec($s) {
  // replace some more - and umlauts
  $patt = array('/&/',   '/</');
  $repl = array('&amp;', '&lt;');
  return htmlify(preg_replace($patt, $repl, $s));
}

function hexfill($o) {
  if(($o & 0xf) == 0)
    return;
  for($i = 0; $i < $o & 0xf; $i++)
    if($i > 0 && $i % 4 == 0)
      echo "    ";
    else
      echo "   ";
}
  
function hexdump($str, $len=0, $offset=0) {

  $a0 = "  ";
  
  if($len === 0)
    $len = strlen($str) - $offset;
  $asc = $a0 . substr("                ", 0, $offset & 0xf);

  hexfill($offset);
  for($i = 0; $i < $len; $i++) {
    $o = $offset + $i;
    if($o > 0 && $o % 16 == 0) {
      echo $asc .PHP_EOL;
      $asc = $a0;
    } else if($o > 0 && $o % 4 == 0)
      echo " ";
    $cc = substr($str, $o, 1);
    $ca = ord($cc);
    $asc .= ($ca > 0x20 && $ca < 0x7f) ? $cc : '.';
    printf("%02x ", $ca);
  }
  hexfill(15 - ($o & 0xf));
  if($asc != $a0) {
    echo $asc .PHP_EOL;
  }
}


function sql_find_pri($xpl) {
  // return primary index of table-explain (table-describe)
  foreach($xpl as $arr) {
    if($arr['Key'] == 'PRI')
      return $arr['Field'];
  }
  return false;
}

function sql_find_fields($xpl, $type) {
  // return array of fields of table-explain matching any of array type
  $ret = array();
  foreach($xpl as $arr)
    foreach($type as $typ)
      if(strstr($arr['Type'], $typ) ) {
	$ret[] = $arr['Field'];
	continue 2;
      }
  return $ret;
}

function sql_find_txtfields($xpl) {
  // return array of textual fields of table-explain
  return sql_find_fields($xpl, array('varchar', 'text', 'mediumtext', 'longtext'));
}

function sql_find_datefields($xpl) {
  // return array of date fields of table-explain
  return sql_find_fields($xpl, array('timestamp', 'datetime', 'date'));
}

function xfind($str, $m1, $m2, $inclusive = 0, $offset = 0) {
  // search string for text surrounded by m1, m2
  // mark1, mark2, incl in [0,1,2]
  $p1 = strpos($str, $m1, $offset ); if($p1 === false) return  false;
  if(!$inclusive) $p1 += strlen($m1);
  $p2 = strpos($str, $m2, $p1); if($p2 === false) return false;
  if($inclusive > 1) $p2 += strlen($m2);
  return substr($str, $p1, $p2-$p1);
}

function linodoc($doc, $bp) {
  // return trimmed line surrounding bp
  for($p1 = $bp; $p1 > 0; $p1--)
    if($doc[$p1] == LF) break;
  $p2 = strpos($doc, LF, $bp);
  if($p2 === false) $p2 = strlen($doc);
  return trim(substr($doc, $p1, $p2-$p1));
}

function nop_is_serialized($s) {
  // check for serialized data
  if(!is_string($s)) return false;
  $b = preg_match('/^a:[0-9]+:{/', $s) === 1;		// array
  $b |= preg_match('/^[sO]:[0-9]+:"/', $s) === 1;	// string or object
  $b |= preg_match('/^[idb]:[0-9]+:[0-9.]/', $s) === 1;	// int, dec, bin (bool)
  return is_string($s) && $b;
}

function nop_subst_serialized($ptrn, $subs, $data) {
  // recursive preg_replace for serialized data
  if(nop_is_serialized($data))
    return serialize(nop_subst_serialized($ptrn, $subs, unserialize($data)));

  switch(gettype($data)) {
  case 'array':
  #case 'object':
    foreach($data as $k => &$v) {
      $v = nop_subst_serialized($ptrn, $subs, $v);
    }
    break;
      
  case 'string':
    $data = preg_replace($ptrn, $subs, $data);
    break;
  }
  return $data;
}

function salt($n = 2) {
  // return random string of length n
  $alpha = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  $alpha .= "abcdefghijklmnopqrstuvwxyz";
  $alpha .= "0123456789";
  $salt = '';
  for($i=0; $i < $n; $i++) {
    $x = 
    $salt .= substr($alpha, mt_rand(0, strlen($alpha)-1), 1);
  }
  $salt = str_shuffle($salt);
  return $salt;
}

function showdiffs($a1, $a2, $an1="Array_1", $an2="Array_2") {
  // display uniques in both arrays
  echo "counts: ". count($a1) .", ". count($a2).LF;
  echo "-- unique to $an1".LF;
  foreach($a1 as $i) { 
    if(!in_array($i, $a2))
      echo $i.LF;
  }
  echo "-- unique to $an2".LF;
  foreach($a2 as $i) { 
    if(!in_array($i, $a1))
      echo $i.LF;
  }
}

function dbg($o, $exit=0) {
  // print conents of o
  if(is_object($o) && count(get_class_methods($o)) > 0) {
    echo get_class($o) . ": ";
    print_r(get_class_methods($o));
  } else if(is_object($o) || is_array($o))
      print_r($o);
  else
    var_dump($o);
  if($exit) die(is_string($exit) ? $exit : '');
}

function hostname($uri) { $a = explode('/', $uri); return $a[1] === '' ? $a[2] : $a[0]; }
function domainname($uri) { return (preg_replace('/^[^.]*\./', '', hostname($uri))); }
  
